Attribute VB_Name = "modUtils"
'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
'  Application Title:       D3DX Texture Converter
'  Developer:               (c)2002 Eric DeBrosse [MS MVP]
'                           The DirectX Cool Fool!
'                           http://www.blown.com/dx/
'
'  File:        modUtils.bas
'  Content:     Helper functions.
'
'               Content is not to be duplicated without permission from
'               the author.  All rights reserved.
'
'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''

Option Explicit

'-----------------------------------------------------------------------------
' Global Variables
'-----------------------------------------------------------------------------

'declare D3D constant missing from DX8.1 VB6 type library
Global Const D3DPOOL_SCRATCH = 3&

'required DirectX objects
Global g_oDX As DirectX8
Global g_oD3D As Direct3D8
Global g_oD3DX As D3DX8
Global g_oD3DDevice As Direct3DDevice8
Global g_d3dpp As D3DPRESENT_PARAMETERS

'used to cancel saving of files if user closes form
Global g_bUserCanceled As Boolean

Global Const SUPPORTED_FILE_TYPES = "All Supported File Types |*.dds;*.tga;*.jpg;*.jpeg;*.jfif;*.jpe;*.bmp;*.dib;*.tif;*.tiff;*.png;*.ppm|Direct Draw Surface (*.dds)|*.dds|Truevision Targa (*.tga)|*.tga|Joint Photographic Experts Group (*.jpg)|*.jpg;*.jpeg;*.jfif;*.jpe|MS-Windows Bitmap (*.bmp)|*.bmp|Device-Independent Bitmap (*.dib)|*.dib|Tag Image File (*.tif)|*.tif;*.tiff|Portable Network Graphics (*.png)|*.png|Portable Pixmap (*.ppm)|*.ppm|All Files (*.*)|*.*"
Global Const SUPPORTED_FILE_PATTERNS = "*.dds;*.tga;*.jpg;*.jpeg;*.jfif;*.jpe;*.bmp;*.dib;*.tif;*.tiff;*.png;*.ppm"

'-----------------------------------------------------------------------------
' Private Win32 API stuff
'-----------------------------------------------------------------------------
Private Declare Function GetFileTitle Lib "comdlg32.dll" Alias "GetFileTitleA" (ByVal lpszFile As String, ByVal lpszTitle As String, ByVal cbBuf As Integer) As Integer

'used to browse for a folder
Private Const BIF_RETURNONLYFSDIRS = 1
'Private Const BIF_DONTGOBELOWDOMAIN = 2
Private Const MAX_PATH = 260

Private Declare Function SHBrowseForFolder Lib "shell32" (lpbi As BrowseInfo) As Long
Private Declare Function SHGetPathFromIDList Lib "shell32" (ByVal pidList As Long, ByVal lpBuffer As String) As Long
Private Declare Function lstrcat Lib "kernel32" Alias "lstrcatA" (ByVal lpString1 As String, ByVal lpString2 As String) As Long

Private Type BrowseInfo
    hWndOwner As Long
    pIDLRoot As Long
    pszDisplayName As Long
    lpszTitle As Long
    ulFlags As Long
    lpfnCallback As Long
    lParam As Long
    iImage As Long
End Type

'-----------------------------------------------------------------------------
' Name: ConvertTexture
' Desc: Opens sSrc file, creates a new texture based on the
'       specifed options and saves the file in the sDest path,
'       using an appropriate filename and extension.  As of
'       DX 8.1, only .DDS and .BMP are supported by the
'       D3DX.SaveTextureToFile function.  If one of the DXT
'       formats are specified, .DDS will be used.  All other
'       file types will be saved as un-compressed .bmp files.
'-----------------------------------------------------------------------------
Public Function ConvertTexture(ByVal sSrc As String, _
                               ByVal sDest As String, _
                               ByVal lWidth As Long, _
                               ByVal lHeight As Long, _
                               ByVal lMipLevels As Long, _
                               ByVal eFormat As CONST_D3DFORMAT, _
                               ByVal eFilter As CONST_D3DXENUM, _
                               ByVal eMipFilter As CONST_D3DXENUM) As String
    
    Dim SrcPalette As PALETTEENTRY
    Dim oTexture As Direct3DTexture8
    Dim eDestFormat As CONST_D3DXIMAGE_FILEFORMAT
    Dim sDestFile As String
    
    On Error Resume Next
    
    sDestFile = sDest
    
    'add filename and extension to destination path
    If eFormat <> D3DFMT_DXT1 _
            And eFormat <> D3DFMT_DXT2 _
            And eFormat <> D3DFMT_DXT3 _
            And eFormat <> D3DFMT_DXT4 _
            And eFormat <> D3DFMT_DXT5 Then
        'needs .bmp extension
        sDestFile = sDestFile & GetFileFromPathEx(sSrc) & ".bmp"
        eDestFormat = D3DXIFF_BMP
    Else
        'needs .dds extension
        sDestFile = sDestFile & GetFileFromPathEx(sSrc) & ".dds"
        eDestFormat = D3DXIFF_DDS
    End If
    
    'create texture in D3DPOOL_SCRATCH to avoid device limitations,
    'this way we can create any texture format we want, without
    'having to create and require a REF device.  Keep in mind with
    'the scratch pool, we can't set this texture on the device or
    'use it as a render target.
    DoEvents
    If g_bUserCanceled = False Then
        On Error GoTo ErrOut
        
        Set oTexture = g_oD3DX.CreateTextureFromFileEx(g_oD3DDevice, _
                sSrc, lWidth, lHeight, lMipLevels, 0&, _
                eFormat, D3DPOOL_SCRATCH, eFilter, eMipFilter, _
                0&, ByVal 0&, ByVal 0&)
    End If
    
    DoEvents
    If g_bUserCanceled = False Then
        'save created texture now
        On Error GoTo ErrOut2
        
        'we must pass am empty SrcPalette paramter to this function
        'or it will fail.  We will use the palette from the file
        'that was loaded.
        g_oD3DX.SaveTextureToFile sDestFile, eDestFormat, oTexture, SrcPalette
    End If
    
    'done saving texture file
    Set oTexture = Nothing
    
    'return filename of texture that was created
    ConvertTexture = sDestFile
Exit Function

ErrOut:
    Debug.Print "Failed to open texture file."
    Set oTexture = Nothing
    ConvertTexture = ""
    Exit Function
    
ErrOut2:
    Debug.Print "Failed to save texture file."
    Set oTexture = Nothing
    ConvertTexture = ""
End Function

'-----------------------------------------------------------------------------
' Name: InitD3D
' Desc: Used to initialize Direct3D using HAL.
'-----------------------------------------------------------------------------
Public Function InitD3D(ByVal hWnd As Long) As Boolean
    On Error GoTo ErrOut
    
    'create DirectX object
    Set g_oDX = New DirectX8
    
    'create the D3D object
    Set g_oD3D = g_oDX.Direct3DCreate()
    If g_oD3D Is Nothing Then
        InitD3D = False
        Exit Function
    End If
    
    'get the current format of the desktop
    'NOTE: device creation will fail if the format of the desktop is 32 bits
    'and the device only has 16 bit D3D modes (i.e. old Voodoo cards)
    Dim DispMode As D3DDISPLAYMODE
    g_oD3D.GetAdapterDisplayMode D3DADAPTER_DEFAULT, DispMode
    
    'set up a generic structure used to create the D3DDevice
    With g_d3dpp
        .Windowed = 1
        .SwapEffect = D3DSWAPEFFECT_FLIP
        .BackBufferFormat = DispMode.Format
        .BackBufferCount = 1
        .EnableAutoDepthStencil = 0
        .AutoDepthStencilFormat = D3DFMT_UNKNOWN
    End With
    
    'create the D3DDevice
    Set g_oD3DDevice = g_oD3D.CreateDevice(D3DADAPTER_DEFAULT, D3DDEVTYPE_HAL, hWnd, _
                                           D3DCREATE_SOFTWARE_VERTEXPROCESSING, g_d3dpp)
    
    If g_oD3DDevice Is Nothing Then
        InitD3D = False
        Exit Function
    End If
    
    'create d3dx object
    Set g_oD3DX = New D3DX8
    
    'device was created
    InitD3D = True
Exit Function

ErrOut:
    'failed to create device
    InitD3D = False
End Function

'-----------------------------------------------------------------------------
' Name: Cleanup
' Desc: Releases all previously initialized objects.
'-----------------------------------------------------------------------------
Public Sub Cleanup()
    Set g_oD3DDevice = Nothing
    Set g_oD3DX = Nothing
    Set g_oD3D = Nothing
    Set g_oDX = Nothing
End Sub

'-----------------------------------------------------------------------------
' Name: BrowseForFolder
' Desc:
'-----------------------------------------------------------------------------
Public Function BrowseForFolder(ByVal DialogTitle As String, ByVal OwnerHwnd As Long) As String
    'open a Treeview control that displays the directories
    Dim lpIDList As Long
    Dim sBuffer As String
    Dim szTitle As String
    Dim tBrowseInfo As BrowseInfo
    
    'set caption on dialog
    szTitle = DialogTitle
    
    With tBrowseInfo
        .hWndOwner = OwnerHwnd
        .lpszTitle = lstrcat(szTitle, "")
        '.ulFlags = BIF_RETURNONLYFSDIRS + BIF_DONTGOBELOWDOMAIN
        .ulFlags = BIF_RETURNONLYFSDIRS
    End With
    
    'show "Browse for Folder" dialog using shell
    lpIDList = SHBrowseForFolder(tBrowseInfo)
    
    If lpIDList <> vbNull Then
        'get folder that was selected
        sBuffer = Space(MAX_PATH)
        SHGetPathFromIDList lpIDList, sBuffer
        sBuffer = Left(sBuffer, InStr(sBuffer, vbNullChar) - 1)
        
        'was a folder selected?
        If sBuffer <> "" Then
            'make sure path ends with a \
            If Right$(sBuffer, 1) <> "\" Then sBuffer = sBuffer & "\"
        End If
        
        'return folder that was selected
        BrowseForFolder = sBuffer
    Else
        'user pressed cancel
        BrowseForFolder = ""
    End If
End Function

'-----------------------------------------------------------------------------
' Name: GetFileFromPath
' Desc: Returns a filename without the path.
'-----------------------------------------------------------------------------
Public Function GetFileFromPath(ByVal strFullPath As String) As String
    Dim hFile As Long
    Dim FileOnly As String
    
    FileOnly = String$(MAX_PATH, Chr(0))
    hFile = GetFileTitle(strFullPath, FileOnly, Len(FileOnly))
    GetFileFromPath = StripNull(FileOnly)
End Function

'-----------------------------------------------------------------------------
' Name: GetFileFromPathEx
' Desc: Returns a filename without the path or an extension.
'-----------------------------------------------------------------------------
Public Function GetFileFromPathEx(ByVal strFullPath As String) As String
    Dim lSpot As Long
    Dim FileOnly As String
    
    'strip path from filename
    FileOnly = GetFileFromPath(strFullPath)
    
    'get the location of the extension
    lSpot = InStr(1, FileOnly, ".", vbTextCompare)
    
    If lSpot <> 0 Then
        GetFileFromPathEx = Left$(FileOnly, lSpot - 1)
    Else
        'there was no extension to remove
        GetFileFromPathEx = FileOnly
    End If
End Function

'-----------------------------------------------------------------------------
' Name: GetPathFromFile
' Desc: Returns just the path to the specified file.
'-----------------------------------------------------------------------------
Public Function GetPathFromFile(ByVal FilePath As String) As String
    On Error Resume Next
    
    GetPathFromFile = Left$(FilePath, Len(FilePath) - Len(GetFileFromPath(FilePath)))
End Function

'-----------------------------------------------------------------------------
' Name: StripNull
' Desc: Internal function, removes Null from the end of a string.
'-----------------------------------------------------------------------------
Private Function StripNull(ByVal strChars As String) As String
    Dim Pos As Integer
    
    Pos = InStr(strChars, Chr$(0))
    
    If Pos > 0 Then
        StripNull = Left$(strChars, Pos - 1)
    Else
        StripNull = strChars
    End If
End Function

